var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArray = (this && this.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};
import React, { useEffect, useRef } from "react";
import { SVG } from "@svgdotjs/svg.js";
import { position, drawTexts, drawLines, drawGrid, drawRects } from "./utils";
var defaultLineStrokeData = { width: 1, color: "#000" };
var defaultMargin = { x: 0, y: 0 };
var defaultRectStrokeData = { width: 1, color: "#000", dasharray: "5,5" };
var defaultTextFontData = { color: "#000" };
var defaultGridStrokeData = { dasharray: "5,5", width: 1, color: "#999" };
var FlowChart = function (props) {
    var nodes = props.nodes, lines = props.lines, _a = props.defaultLineStroke, defaultLineStroke = _a === void 0 ? defaultLineStrokeData : _a, _b = props.colWidth, colWidth = _b === void 0 ? 150 : _b, _c = props.rowHeight, rowHeight = _c === void 0 ? 100 : _c, _d = props.nodeWidth, nodeWidth = _d === void 0 ? 120 : _d, _e = props.nodeHeight, nodeHeight = _e === void 0 ? 80 : _e, _f = props.margin, margin = _f === void 0 ? defaultMargin : _f, grid = props.grid, rects = props.rects, _g = props.defaultRectStroke, defaultRectStroke = _g === void 0 ? defaultRectStrokeData : _g, texts = props.texts, _h = props.defaultTextFont, defaultTextFont = _h === void 0 ? defaultTextFontData : _h, _j = props.dis, dis = _j === void 0 ? 10 : _j, _k = props.hoverLineColor, hoverLineColor = _k === void 0 ? "#000" : _k, _l = props.defaultGridStroke, defaultGridStroke = _l === void 0 ? defaultGridStrokeData : _l, children = props.children, render = props.render;
    var renderNodes = function () {
        return nodes.map(function (node) {
            var childrenComponents = render
                ? undefined
                : React.Children.map(children, function (child) {
                    if (React.isValidElement(child)) {
                        return React.cloneElement(child, { node: __assign({}, node) });
                    }
                });
            return (React.createElement("div", { className: "flow-node", style: position(node, margin, colWidth, rowHeight, nodeWidth, nodeHeight), key: node.id },
                render && render(node),
                childrenComponents));
        });
    };
    var wrapper = useRef(null);
    var draw = useRef(SVG());
    useEffect(function () {
        draw.current.addTo(wrapper.current).size("100%", "100%");
    }, []);
    useEffect(function () {
        var _a, _b, _c, _d;
        var current = draw.current;
        // 保存每一列的节点
        var cols = [];
        // 保存每一行的节点
        var rows = [];
        // 箭头大小
        var arrowSize = 5;
        // 收集每一行节点数，节点按列排序
        var tempNodes = __spreadArray([], nodes);
        tempNodes.sort(function (a, b) { return a.col - b.col; });
        tempNodes.forEach(function (_a) {
            var id = _a.id, row = _a.row;
            if (typeof rows[row] === "undefined") {
                rows[row] = [];
            }
            rows[row].push(id);
        });
        // 收集每一列的节点数，节点间按行排列
        tempNodes.sort(function (a, b) { return a.col - b.col; });
        tempNodes.forEach(function (_a) {
            var id = _a.id, col = _a.col;
            if (typeof cols[col] === "undefined") {
                cols[col] = [];
            }
            cols[col].push(id);
        });
        drawLines(current, lines, arrowSize, hoverLineColor, defaultLineStroke, nodes, dis, cols, rows, colWidth, rowHeight, nodeHeight, nodeWidth, margin);
        if (grid) {
            drawGrid(current, ((_a = wrapper.current) === null || _a === void 0 ? void 0 : _a.offsetWidth) || 0, ((_b = wrapper.current) === null || _b === void 0 ? void 0 : _b.offsetHeight) || 0, grid, colWidth, rowHeight, defaultGridStroke);
        }
        drawTexts(current, texts || [], defaultTextFont, defaultGridStroke, rowHeight, colWidth);
        drawRects(current, rects || [], ((_c = wrapper.current) === null || _c === void 0 ? void 0 : _c.offsetWidth) || 0, ((_d = wrapper.current) === null || _d === void 0 ? void 0 : _d.offsetHeight) || 0, colWidth, rowHeight, defaultRectStroke);
        return function () {
            current.clear();
        };
    }, [
        lines,
        grid,
        draw,
        rects,
        defaultGridStroke,
        defaultLineStroke,
        colWidth,
        rowHeight,
        hoverLineColor,
        defaultRectStroke,
        defaultTextFont,
        texts,
        nodeWidth,
        nodeHeight,
        dis,
        margin,
        nodes,
    ]);
    return (React.createElement("div", { className: "wrapper", ref: wrapper }, renderNodes()));
};
export default FlowChart;
